from abc import ABC, abstractmethod
from typing import Dict, Optional, Any, Union

from imecilabt.gpulab.model.job import Job, JobState
from imecilabt.gpulab.model.job_filter import JobFilter

# Master: exposes all functionality of JobController, that is needed by the slave
from imecilabt.gpulab.model.slave_info import SlaveInfo


class Master(ABC):

    @abstractmethod
    def find_and_start_job(self, job_filter: JobFilter, slave_id: Optional[Any] = None) -> Optional[Job]:
        pass

    @abstractmethod
    def update_job_state(self, job_id: str, target_job_state: JobState, onlyif_current_state: Optional[JobState] = None) -> None:
        pass

    @abstractmethod
    def update_job_run_details(self, job_id: str, run_details: Dict) -> None:
        pass

    @abstractmethod
    def update_job_run_details_field(self, job_id: str, field_name: str, field_new_value: Any) -> None:
        pass

    @abstractmethod
    def get_job(self, job_id: str) -> Job:
        pass

    @abstractmethod
    def append_to_log(self, job_id: str, extra_content: Union[bytes, str]) -> None:
        pass

    #Predefined logging levels are ints mapping to: CRITICAL, ERROR, WARNING, INFO, DEBUG
    @abstractmethod
    def register_logging_event(self, job_id: str, level: int, msg: str) -> None:
        pass

    @abstractmethod
    def report_slave_info(self, slave_info: SlaveInfo) -> None:
        pass
